@@ -19,23 +19,27 @@
 
 #include "postgres.h"
 
-#include "access/heapam.h"
+#include "access/sysattr.h"
 #include "access/htup_details.h"
 #include "access/multixact.h"
-#include "access/table.h"
 #include "access/xact.h"
+#include "storage/bufmgr.h"
 #include "executor/tuptable.h"
 #include "nodes/execnodes.h"
 #include "nodes/extensible.h"
 #include "nodes/nodes.h"
 #include "nodes/plannodes.h"
 #include "parser/parsetree.h"
-#include "storage/bufmgr.h"
+#include "parser/parse_relation.h"
+#include "rewrite/rewriteHandler.h"
 #include "utils/rel.h"
+#include "utils/tqual.h"
 
 #include "catalog/ag_label.h"
+#include "commands/label_commands.h"
 #include "executor/cypher_executor.h"
 #include "executor/cypher_utils.h"
+#include "parser/cypher_parse_node.h"
 #include "nodes/cypher_nodes.h"
 #include "utils/agtype.h"
 #include "utils/graphid.h"
@@ -48,9 +52,7 @@ static void rescan_cypher_delete(CustomScanState *node);
 
 static void process_delete_list(CustomScanState *node);
 
-static void find_connected_edges(CustomScanState *node, char *graph_name,
-                                 List *labels, char *var_name, graphid id,
-                                 bool detach_delete);
+static void check_for_connected_edges(CustomScanState *node);
 static agtype_value *extract_entity(CustomScanState *node,
                                     TupleTableSlot *scanTupleSlot,
                                     int entity_position);
@@ -83,6 +85,7 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
     cypher_delete_custom_scan_state *css =
         (cypher_delete_custom_scan_state *)node;
     Plan *subplan;
+    HASHCTL hashctl;
 
     Assert(list_length(css->cs->custom_plans) == 1);
 
@@ -95,8 +98,7 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
 
     // setup scan tuple slot and projection info
     ExecInitScanTupleSlot(estate, &node->ss,
-                          ExecGetResultType(node->ss.ps.lefttree),
-                          &TTSOpsHeapTuple);
+                          ExecGetResultType(node->ss.ps.lefttree));
 
     if (!CYPHER_CLAUSE_IS_TERMINAL(css->flags))
     {
@@ -112,6 +114,16 @@ static void begin_cypher_delete(CustomScanState *node, EState *estate,
      */
     css->edge_labels = get_all_edge_labels_per_graph(estate, css->delete_data->graph_oid);
 
+    /* init vertex_id_htab */
+    MemSet(&hashctl, 0, sizeof(hashctl));
+    hashctl.keysize = sizeof(graphid);
+    hashctl.entrysize =
+        sizeof(graphid); // entries are not used, but entrysize must >= keysize
+    hashctl.hash = tag_hash;
+    css->vertex_id_htab = hash_create(DELETE_VERTEX_HTAB_NAME,
+                                      DELETE_VERTEX_HTAB_SIZE, &hashctl,
+                                      HASH_ELEM | HASH_FUNCTION);
+
     /*
      * Postgres does not assign the es_output_cid in queries that do
      * not write to disk, ie: SELECT commands. We need the command id
@@ -194,6 +206,10 @@ static TupleTableSlot *exec_cypher_delete(CustomScanState *node)
  */
 static void end_cypher_delete(CustomScanState *node)
 {
+    check_for_connected_edges(node);
+
+    hash_destroy(((cypher_delete_custom_scan_state *)node)->vertex_id_htab);
+
     ExecEndNode(node->ss.ps.lefttree);
 }
 
@@ -279,16 +295,16 @@ static agtype_value *extract_entity(CustomScanState *node,
 static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
                           HeapTuple tuple)
 {
-    ResultRelInfo **saved_resultRels;
+    ResultRelInfo *saved_resultRelInfo;
     LockTupleMode lockmode;
-    TM_FailureData hufd;
-    TM_Result lock_result;
-    TM_Result delete_result;
+    HeapUpdateFailureData hufd;
+    HTSU_Result lock_result;
+    HTSU_Result delete_result;
     Buffer buffer;
 
     // Find the physical tuple, this variable is coming from
-    saved_resultRels = estate->es_result_relations;
-    estate->es_result_relations = &resultRelInfo;
+    saved_resultRelInfo = estate->es_result_relation_info;
+    estate->es_result_relation_info = resultRelInfo;
 
     lockmode = ExecUpdateLockMode(estate, resultRelInfo);
 
@@ -300,11 +316,11 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
      * It is possible the entity may have already been deleted. If the tuple
      * can be deleted, the lock result will be HeapTupleMayBeUpdated. If the
      * tuple was already deleted by this DELETE clause, the result would be
-     * TM_SelfModified, if the result was deleted by a previous delete
-     * clause, the result will TM_Invisible. Throw an error if any
+     * HeapTupleSelfUpdated, if the result was deleted by a previous delete
+     * clause, the result will HeapTupleInvisible. Throw an error if any
      * other result was returned.
      */
-    if (lock_result == TM_Ok)
+    if (lock_result == HeapTupleMayBeUpdated)
     {
         delete_result = heap_delete(resultRelInfo->ri_RelationDesc,
                                     &tuple->t_self, GetCurrentCommandId(true),
@@ -317,32 +333,30 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
          */
         switch (delete_result)
         {
-        case TM_Ok:
-            break;
-        case TM_SelfModified:
-            ereport(
-                ERROR,
-                (errcode(ERRCODE_INTERNAL_ERROR),
-                 errmsg(
-                     "deleting the same entity more than once cannot happen")));
-            /* ereport never gets here */
-            break;
-        case TM_Updated:
-            ereport(
-                ERROR,
-                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
-                 errmsg("could not serialize access due to concurrent update")));
-            /* ereport never gets here */
-            break;
-        default:
-            elog(ERROR, "Entity failed to be update");
-            /* elog never gets here */
-            break;
+                case HeapTupleMayBeUpdated:
+                        break;
+                case HeapTupleSelfUpdated:
+                        ereport(ERROR,
+                                (errcode(ERRCODE_INTERNAL_ERROR),
+                                         errmsg("deleting the same entity more than once cannot happen")));
+                        /* ereport never gets here */
+                        break;
+                case HeapTupleUpdated:
+                        ereport(ERROR,
+                                (errcode(ERRCODE_T_R_SERIALIZATION_FAILURE),
+                                         errmsg("could not serialize access due to concurrent update")));
+                        /* ereport never gets here */
+                        break;
+                default:
+                        elog(ERROR, "Entity failed to be update");
+                        /* elog never gets here */
+                        break;
         }
         /* increment the command counter */
         CommandCounterIncrement();
     }
-    else if (lock_result != TM_Invisible && lock_result != TM_SelfModified)
+    else if (lock_result != HeapTupleInvisible &&
+             lock_result != HeapTupleSelfUpdated)
     {
         ereport(ERROR,
                 (errcode(ERRCODE_INTERNAL_ERROR),
@@ -352,7 +366,7 @@ static void delete_entity(EState *estate, ResultRelInfo *resultRelInfo,
 
     ReleaseBuffer(buffer);
 
-    estate->es_result_relations = saved_resultRels;
+    estate->es_result_relation_info = saved_resultRelInfo;
 }
 
 /*
@@ -373,7 +387,7 @@ static void process_delete_list(CustomScanState *node)
         cypher_delete_item *item;
         agtype_value *original_entity_value, *id, *label;
         ScanKeyData scan_keys[1];
-        TableScanDesc scan_desc;
+        HeapScanDesc scan_desc;
         ResultRelInfo *resultRelInfo;
         HeapTuple heap_tuple;
         char *label_name;
@@ -423,8 +437,10 @@ static void process_delete_list(CustomScanState *node)
         /*
          * Setup the scan description, with the correct snapshot and scan keys.
          */
-        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
-                                    estate->es_snapshot, 1, scan_keys);
+        estate->es_snapshot->curcid = GetCurrentCommandId(false);
+        estate->es_output_cid = GetCurrentCommandId(false);
+        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
+                                   estate->es_snapshot, 1, scan_keys);
 
         /* Retrieve the tuple. */
         heap_tuple = heap_getnext(scan_desc, ForwardScanDirection);
@@ -436,113 +452,119 @@ static void process_delete_list(CustomScanState *node)
          */
         if (!HeapTupleIsValid(heap_tuple))
         {
-            table_endscan(scan_desc);
+            heap_endscan(scan_desc);
             destroy_entity_result_rel_info(resultRelInfo);
 
             continue;
         }
 
         /*
-         * For vertices, we need to check if the vertex is connected to any
-         * edges, * if there are, we need to delete them or throw an error,
-         * depending on if the query specified the DETACH option.
+         * For vertices, we insert the vertex ID in the hashtable
+         * vertex_id_htab. This hashtable is used later to process
+         * connected edges.
          */
         if (original_entity_value->type == AGTV_VERTEX)
         {
-            find_connected_edges(node, css->delete_data->graph_name,
-                                 css->edge_labels, item->var_name,
-                                 id->val.int_value, css->delete_data->detach);
+            bool found;
+            hash_search(css->vertex_id_htab, (void *)&(id->val.int_value),
+                        HASH_ENTER, &found);
         }
 
         /* At this point, we are ready to delete the node/vertex. */
         delete_entity(estate, resultRelInfo, heap_tuple);
 
         /* Close the scan and the relation. */
-        table_endscan(scan_desc);
+        heap_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
 }
 
 /*
- * Find the edges connected to the given node. If there is any edges either
- * delete them or throw an error, depending on the detach delete option.
+ * Scans the edge tables and checks if the deleted vertices are connected to
+ * any edge(s). For DETACH DELETE, the connected edges are deleted. Otherwise,
+ * an error is thrown.
  */
-static void find_connected_edges(CustomScanState *node, char *graph_name,
-                                 List *labels, char *var_name, graphid id,
-                                 bool detach_delete)
+static void check_for_connected_edges(CustomScanState *node)
 {
+    ListCell *lc;
     cypher_delete_custom_scan_state *css =
         (cypher_delete_custom_scan_state *)node;
     EState *estate = css->css.ss.ps.state;
-    ListCell *lc;
+    char *graph_name = css->delete_data->graph_name;
 
-    Increment_Estate_CommandId(estate);
-
-    /*
-     * We need to scan through all the edges to see if this vertex has
-     * any edges attached to it.
-     *
-     * XXX: If we implement an on-disc graph storage system. Such as
-     * an adjacency matrix, the performance of this check can be massively
-     * improved. However, right now we have to scan every edge to see if
-     * one has this vertex as a start or end vertex.
-     */
-    foreach(lc, labels)
+    /* scans each label from css->edge_labels */
+    foreach (lc, css->edge_labels)
     {
         char *label_name = lfirst(lc);
         ResultRelInfo *resultRelInfo;
-        TableScanDesc scan_desc;
+        HeapScanDesc scan_desc;
         HeapTuple tuple;
         TupleTableSlot *slot;
 
         resultRelInfo = create_entity_result_rel_info(estate,
                                                       graph_name, label_name);
 
-        scan_desc = table_beginscan(resultRelInfo->ri_RelationDesc,
-                                    estate->es_snapshot, 0, NULL);
+        estate->es_snapshot->curcid = GetCurrentCommandId(false);
+        estate->es_output_cid = GetCurrentCommandId(false);
 
-        slot = ExecInitExtraTupleSlot(
-            estate, RelationGetDescr(resultRelInfo->ri_RelationDesc),
-            &TTSOpsHeapTuple);
+        scan_desc = heap_beginscan(resultRelInfo->ri_RelationDesc,
+                                   estate->es_snapshot, 0, NULL);
 
-        // scan the table
-        while(true)
+        slot = ExecInitExtraTupleSlot(estate,
+                    RelationGetDescr(resultRelInfo->ri_RelationDesc));
+
+        /* for each row */
+        while (true)
         {
-            graphid startid, endid;
+            graphid startid;
+            graphid endid;
             bool isNull;
+            bool found_startid = false;
+            bool found_endid = false;
 
             tuple = heap_getnext(scan_desc, ForwardScanDirection);
 
-            // no more tuples to process, break and scan the next label.
+            /* no more tuples to process, break and scan the next label. */
             if (!HeapTupleIsValid(tuple))
-                break;
-
-            ExecStoreHeapTuple(tuple, slot, false);
-
-            startid = GRAPHID_GET_DATUM(slot_getattr(slot, Anum_ag_label_edge_table_start_id, &isNull));
-            endid = GRAPHID_GET_DATUM(slot_getattr(slot, Anum_ag_label_edge_table_end_id, &isNull));
-
-            if (id == startid || id == endid)
             {
-                /*
-                 * We have found an edge that uses the vertex. Either delete the
-                 * edge or throw an error. Depending on whether the DETACH
-                 * option was specified in the query.
-                 */
-                if (detach_delete)
+                break;
+            }
+
+            ExecStoreTuple(tuple, slot, InvalidBuffer, false);
+
+            startid = GRAPHID_GET_DATUM(slot_getattr(
+                slot, Anum_ag_label_edge_table_start_id, &isNull));
+            endid = GRAPHID_GET_DATUM(
+                slot_getattr(slot, Anum_ag_label_edge_table_end_id, &isNull));
+
+            hash_search(css->vertex_id_htab, (void *)&startid, HASH_FIND,
+                        &found_startid);
+
+            if (!found_startid)
+            {
+                hash_search(css->vertex_id_htab, (void *)&endid, HASH_FIND,
+                            &found_endid);
+            }
+
+            if (found_startid || found_endid)
+            {
+                if (css->delete_data->detach)
+                {
                     delete_entity(estate, resultRelInfo, tuple);
+                }
                 else
-                    ereport(ERROR,
-                            (errcode(ERRCODE_INTERNAL_ERROR),
-                             errmsg("Cannot delete vertex %s, because it still has edges attached. "
-                                    "To delete this vertex, you must first delete the attached edges.",
-                                    var_name)));
+                {
+                    ereport(
+                        ERROR,
+                        (errcode(ERRCODE_INTERNAL_ERROR),
+                         errmsg(
+                             "Cannot delete a vertex that has edge(s). "
+                             "Delete the edge(s) first, or try DETACH DELETE.")));
+                }
             }
         }
 
-        table_endscan(scan_desc);
+        heap_endscan(scan_desc);
         destroy_entity_result_rel_info(resultRelInfo);
     }
-
-    Decrement_Estate_CommandId(estate);
 }
