@@ -24,36 +24,68 @@
 
 #include "postgres.h"
 
-#include "access/attnum.h"
-#include "access/htup.h"
-#include "access/tupdesc.h"
-#include "catalog/pg_type.h"
+#include "catalog/pg_type_d.h"
 #include "common/keywords.h"
-#include "fmgr.h"
 #include "funcapi.h"
 
 #include "parser/cypher_gram.h"
-#include "parser/cypher_kwlist_d.h"
 
 /*
  * This list must be sorted by ASCII name, because binary search is used to
  * locate entries.
  */
-#define PG_KEYWORD(kwname, value, category) value,
-
-const uint16 CypherKeywordTokens[] = {
-#include "parser/cypher_kwlist.h"
+const ScanKeyword cypher_keywords[] = {
+    {"all", ALL, RESERVED_KEYWORD},
+    {"analyze", ANALYZE, RESERVED_KEYWORD},
+    {"and", AND, RESERVED_KEYWORD},
+    {"as", AS, RESERVED_KEYWORD},
+    {"asc", ASC, RESERVED_KEYWORD},
+    {"ascending", ASCENDING, RESERVED_KEYWORD},
+    {"by", BY, RESERVED_KEYWORD},
+    {"call", CALL, RESERVED_KEYWORD},
+    {"case", CASE, RESERVED_KEYWORD},
+    {"coalesce", COALESCE, RESERVED_KEYWORD},
+    {"contains", CONTAINS, RESERVED_KEYWORD},
+    {"create", CREATE, RESERVED_KEYWORD},
+    {"delete", DELETE, RESERVED_KEYWORD},
+    {"desc", DESC, RESERVED_KEYWORD},
+    {"descending", DESCENDING, RESERVED_KEYWORD},
+    {"detach", DETACH, RESERVED_KEYWORD},
+    {"distinct", DISTINCT, RESERVED_KEYWORD},
+    {"else", ELSE, RESERVED_KEYWORD},
+    {"end", END_P, RESERVED_KEYWORD},
+    {"ends", ENDS, RESERVED_KEYWORD},
+    {"exists", EXISTS, RESERVED_KEYWORD},
+    {"explain", EXPLAIN, RESERVED_KEYWORD},
+    {"false", FALSE_P, RESERVED_KEYWORD},
+    {"in", IN, RESERVED_KEYWORD},
+    {"is", IS, RESERVED_KEYWORD},
+    {"limit", LIMIT, RESERVED_KEYWORD},
+    {"match", MATCH, RESERVED_KEYWORD},
+    {"merge", MERGE, RESERVED_KEYWORD},
+    {"not", NOT, RESERVED_KEYWORD},
+    {"null", NULL_P, RESERVED_KEYWORD},
+    {"optional", OPTIONAL, RESERVED_KEYWORD},
+    {"or", OR, RESERVED_KEYWORD},
+    {"order", ORDER, RESERVED_KEYWORD},
+    {"remove", REMOVE, RESERVED_KEYWORD},
+    {"return", RETURN, RESERVED_KEYWORD},
+    {"set", SET, RESERVED_KEYWORD},
+    {"skip", SKIP, RESERVED_KEYWORD},
+    {"starts", STARTS, RESERVED_KEYWORD},
+    {"then", THEN, RESERVED_KEYWORD},
+    {"true", TRUE_P, RESERVED_KEYWORD},
+    {"union", UNION, RESERVED_KEYWORD},
+    {"unwind", UNWIND, RESERVED_KEYWORD},
+    {"verbose", VERBOSE, RESERVED_KEYWORD},
+    {"when", WHEN, RESERVED_KEYWORD},
+    {"where", WHERE, RESERVED_KEYWORD},
+    {"with", WITH, RESERVED_KEYWORD},
+    {"xor", XOR, RESERVED_KEYWORD},
+    {"yield", YIELD, RESERVED_KEYWORD}
 };
 
-#undef PG_KEYWORD
-
-#define PG_KEYWORD(kwname, value, category) category,
-
-const uint16 CypherKeywordCategories[] = {
-#include "parser/cypher_kwlist.h"
-};
-
-#undef PG_KEYWORD
+const int num_cypher_keywords = lengthof(cypher_keywords);
 
 PG_FUNCTION_INFO_V1(get_cypher_keywords);
 
@@ -70,7 +102,7 @@ Datum get_cypher_keywords(PG_FUNCTION_ARGS)
         func_ctx = SRF_FIRSTCALL_INIT();
         old_mem_ctx = MemoryContextSwitchTo(func_ctx->multi_call_memory_ctx);
 
-        tup_desc = CreateTemplateTupleDesc(3);
+        tup_desc = CreateTemplateTupleDesc(3, false);
         TupleDescInitEntry(tup_desc, (AttrNumber)1, "word", TEXTOID, -1, 0);
         TupleDescInitEntry(tup_desc, (AttrNumber)2, "catcode", CHAROID, -1, 0);
         TupleDescInitEntry(tup_desc, (AttrNumber)3, "catdesc", TEXTOID, -1, 0);
@@ -82,16 +114,15 @@ Datum get_cypher_keywords(PG_FUNCTION_ARGS)
 
     func_ctx = SRF_PERCALL_SETUP();
 
-    if (func_ctx->call_cntr < CypherKeyword.num_keywords)
+    if (func_ctx->call_cntr < num_cypher_keywords)
     {
         char *values[3];
         HeapTuple tuple;
 
         // cast-away-const is ugly but alternatives aren't much better
-        values[0] = (char *) GetScanKeyword((int) func_ctx->call_cntr,
-                                            &CypherKeyword);
+        values[0] = (char *)cypher_keywords[func_ctx->call_cntr].name;
 
-        switch (CypherKeywordCategories[func_ctx->call_cntr])
+        switch (cypher_keywords[func_ctx->call_cntr].category)
         {
         case UNRESERVED_KEYWORD:
             values[1] = "U";
