@@ -19,29 +19,17 @@
 
 #include "postgres.h"
 
-#include "access/heapam.h"
-#include "access/relscan.h"
-#include "access/skey.h"
-#include "access/table.h"
-#include "access/tableam.h"
 #include "catalog/namespace.h"
-#include "commands/label_commands.h"
+#include "utils/datum.h"
 #include "utils/lsyscache.h"
 #include "utils/memutils.h"
 #include "utils/rel.h"
 #include "utils/snapmgr.h"
 #include "commands/label_commands.h"
-#include "common/hashfn.h"
 
-#include "catalog/ag_graph.h"
-#include "catalog/ag_label.h"
 #include "utils/age_global_graph.h"
-#include "utils/age_graphid_ds.h"
-#include "utils/agtype.h"
 #include "catalog/ag_graph.h"
 #include "catalog/ag_label.h"
-#include "utils/graphid.h"
-#include "utils/age_graphid_ds.h"
 
 /* defines */
 #define VERTEX_HTAB_NAME "Vertex to edge lists " /* added a space at end for */
@@ -199,7 +187,7 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
     List *labels = NIL;
     ScanKeyData scan_keys[2];
     Relation ag_label;
-    TableScanDesc scan_desc;
+    HeapScanDesc scan_desc;
     HeapTuple tuple;
     TupleDesc tupdesc;
 
@@ -213,8 +201,8 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
                 F_CHAREQ, CharGetDatum(label_type));
 
     /* setup the table to be scanned, ag_label in this case */
-    ag_label = table_open(ag_label_relation_id(), ShareLock);
-    scan_desc = table_beginscan(ag_label, snapshot, 2, scan_keys);
+    ag_label = heap_open(ag_label_relation_id(), ShareLock);
+    scan_desc = heap_beginscan(ag_label, snapshot, 2, scan_keys);
 
     /* get the tupdesc - we don't need to release this one */
     tupdesc = RelationGetDescr(ag_label);
@@ -238,8 +226,8 @@ static List *get_ag_labels_names(Snapshot snapshot, Oid graph_oid,
     }
 
     /* close up scan */
-    table_endscan(scan_desc);
-    table_close(ag_label, ShareLock);
+    heap_endscan(scan_desc);
+    heap_close(ag_label, ShareLock);
 
     return labels;
 }
@@ -409,7 +397,7 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
     foreach (lc, vertex_label_names)
     {
         Relation graph_vertex_label;
-        TableScanDesc scan_desc;
+        HeapScanDesc scan_desc;
         HeapTuple tuple;
         char *vertex_label_name;
         Oid vertex_label_table_oid;
@@ -421,8 +409,8 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
         vertex_label_table_oid = get_relname_relid(vertex_label_name,
                                                    graph_namespace_oid);
         /* open the relation (table) and begin the scan */
-        graph_vertex_label = table_open(vertex_label_table_oid, ShareLock);
-        scan_desc = table_beginscan(graph_vertex_label, snapshot, 0, NULL);
+        graph_vertex_label = heap_open(vertex_label_table_oid, ShareLock);
+        scan_desc = heap_beginscan(graph_vertex_label, snapshot, 0, NULL);
         /* get the tupdesc - we don't need to release this one */
         tupdesc = RelationGetDescr(graph_vertex_label);
         /* bail if the number of columns differs */
@@ -448,6 +436,8 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
             /* get the vertex properties datum */
             vertex_properties = column_get_datum(tupdesc, tuple, 1,
                                                  "properties", AGTYPEOID, true);
+            /* we need to make a copy of the properties datum */
+            vertex_properties = datumCopy(vertex_properties, false, -1);
 
             /* insert vertex into vertex hashtable */
             inserted = insert_vertex_entry(ggctx, vertex_id,
@@ -462,8 +452,8 @@ static void load_vertex_hashtable(GRAPH_global_context *ggctx)
         }
 
         /* end the scan and close the relation */
-        table_endscan(scan_desc);
-        table_close(graph_vertex_label, ShareLock);
+        heap_endscan(scan_desc);
+        heap_close(graph_vertex_label, ShareLock);
     }
 }
 
@@ -508,7 +498,7 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
     foreach (lc, edge_label_names)
     {
         Relation graph_edge_label;
-        TableScanDesc scan_desc;
+        HeapScanDesc scan_desc;
         HeapTuple tuple;
         char *edge_label_name;
         Oid edge_label_table_oid;
@@ -520,8 +510,8 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
         edge_label_table_oid = get_relname_relid(edge_label_name,
                                                  graph_namespace_oid);
         /* open the relation (table) and begin the scan */
-        graph_edge_label = table_open(edge_label_table_oid, ShareLock);
-        scan_desc = table_beginscan(graph_edge_label, snapshot, 0, NULL);
+        graph_edge_label = heap_open(edge_label_table_oid, ShareLock);
+        scan_desc = heap_beginscan(graph_edge_label, snapshot, 0, NULL);
         /* get the tupdesc - we don't need to release this one */
         tupdesc = RelationGetDescr(graph_edge_label);
         /* bail if the number of columns differs */
@@ -561,6 +551,9 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
             edge_properties = column_get_datum(tupdesc, tuple, 3, "properties",
                                                AGTYPEOID, true);
 
+            /* we need to make a copy of the properties datum */
+            edge_properties = datumCopy(edge_properties, false, -1);
+
             /* insert edge into edge hashtable */
             inserted = insert_edge(ggctx, edge_id, edge_properties,
                                    edge_vertex_start_id, edge_vertex_end_id,
@@ -583,8 +576,8 @@ static void load_edge_hashtable(GRAPH_global_context *ggctx)
         }
 
         /* end the scan and close the relation */
-        table_endscan(scan_desc);
-        table_close(graph_edge_label, ShareLock);
+        heap_endscan(scan_desc);
+        heap_close(graph_edge_label, ShareLock);
     }
 }
 
