@@ -19,7 +19,6 @@
 
 #include "postgres.h"
 
-#include "fmgr.h"
 #include "utils/builtins.h"
 #include "utils/sortsupport.h"
 
@@ -36,8 +35,7 @@ Oid get_GRAPHIDOID(void)
 {
     if (g_GRAPHIDOID == InvalidOid)
     {
-        g_GRAPHIDOID = GetSysCacheOid2(TYPENAMENSP, Anum_pg_type_oid,
-                                       CStringGetDatum("graphid"),
+        g_GRAPHIDOID = GetSysCacheOid2(TYPENAMENSP, CStringGetDatum("graphid"),
                                        ObjectIdGetDatum(ag_catalog_namespace_id()));
     }
 
@@ -49,7 +47,7 @@ Oid get_GRAPHIDARRAYOID(void)
 {
     if (g_GRAPHIDARRAYOID == InvalidOid)
     {
-        g_GRAPHIDARRAYOID = GetSysCacheOid2(TYPENAMENSP, Anum_pg_type_oid,
+        g_GRAPHIDARRAYOID = GetSysCacheOid2(TYPENAMENSP,
                                             CStringGetDatum("_graphid"),
                                             ObjectIdGetDatum(ag_catalog_namespace_id()));
     }
