@@ -19,31 +19,16 @@
 
 #include "postgres.h"
 
-#include "access/attnum.h"
 #include "access/genam.h"
 #include "access/heapam.h"
-#include "access/htup.h"
-#include "access/htup_details.h"
-#include "access/skey.h"
-#include "access/stratnum.h"
-#include "access/tupdesc.h"
 #include "catalog/pg_collation.h"
-#include "fmgr.h"
-#include "storage/lockdefs.h"
 #include "utils/builtins.h"
 #include "utils/catcache.h"
-#include "utils/fmgroids.h"
-#include "utils/hsearch.h"
 #include "utils/inval.h"
-#include "utils/memutils.h"
-#include "utils/rel.h"
-#include "utils/relcache.h"
-#include "utils/syscache.h"
 
 #include "catalog/ag_graph.h"
 #include "catalog/ag_label.h"
 #include "utils/ag_cache.h"
-#include "utils/graphid.h"
 
 typedef struct graph_name_cache_entry
 {
@@ -534,6 +519,12 @@ static void initialize_label_caches(void)
     ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[1],
                            Anum_ag_label_graph, F_OIDEQ);
 
+    // ag_label.seq_name, ag_label.graph
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[0],
+                           Anum_ag_label_seq_name, F_NAMEEQ);
+    ag_cache_scan_key_init(&label_seq_name_graph_scan_keys[1],
+                           Anum_ag_label_graph, F_OIDEQ);
+
     create_label_caches();
 
     /*
@@ -1159,7 +1150,7 @@ static void *label_seq_name_graph_cache_hash_search(Name name, Oid graph,
     label_seq_name_graph_cache_key key;
 
     // initialize the hash key for label_seq_name_graph_cache_hash
-    namestrcpy(&key.name, name->data);
+    namecpy(&key.name, name);
     key.graph = graph;
 
     return hash_search(label_seq_name_graph_cache_hash, &key, action, found);
@@ -1197,5 +1188,5 @@ static void fill_label_cache_data(label_cache_data *cache_data,
     // ag_label.seq_name
     value = heap_getattr(tuple, Anum_ag_label_seq_name, tuple_desc, &is_null);
     Assert(!is_null);
-    namestrcpy(&cache_data->seq_name, DatumGetName(value)->data);
+    namecpy(&cache_data->seq_name, DatumGetName(value));
 }
