@@ -16,50 +16,8 @@
  * specific language governing permissions and limitations
  * under the License.
  */
-
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-#include <stdlib.h>
-#include <unistd.h>
-
 #include "postgres.h"
 
-#include "access/heapam.h"
-#include "access/xact.h"
-#include "catalog/dependency.h"
-#include "catalog/namespace.h"
-#include "catalog/objectaddress.h"
-#include "catalog/pg_class_d.h"
-#include "commands/defrem.h"
-#include "commands/sequence.h"
-#include "commands/tablecmds.h"
-#include "miscadmin.h"
-#include "nodes/makefuncs.h"
-#include "nodes/nodes.h"
-#include "nodes/parsenodes.h"
-#include "nodes/pg_list.h"
-#include "nodes/plannodes.h"
-#include "nodes/primnodes.h"
-#include "nodes/value.h"
-#include "parser/parse_node.h"
-#include "parser/parser.h"
-#include "storage/lockdefs.h"
-#include "tcop/dest.h"
-#include "tcop/utility.h"
-#include "utils/acl.h"
-#include "utils/builtins.h"
-#include "utils/inval.h"
-#include "utils/lsyscache.h"
-#include "utils/rel.h"
-
-#include "catalog/ag_graph.h"
-#include "catalog/ag_label.h"
-#include "commands/label_commands.h"
-#include "utils/ag_cache.h"
-#include "utils/agtype.h"
-#include "utils/graphid.h"
-
 #include "utils/load/ag_load_labels.h"
 #include "utils/load/age_load.h"
 #include "utils/load/csv.h"
@@ -135,7 +93,7 @@ void vertex_row_cb(int delim __attribute__((unused)), void *data)
 
         props = create_agtype_from_list(cr->header, cr->fields,
                                         n_fields, label_id_int);
-        insert_vertex_simple(cr->graph_oid, cr->object_name,
+        insert_vertex_simple(cr->graph_id, cr->object_name,
                              object_graph_id, props);
         pfree(props);
     }
@@ -184,7 +142,7 @@ static int is_term(unsigned char c)
 
 int create_labels_from_csv_file(char *file_path,
                                 char *graph_name,
-                                Oid graph_oid,
+                                Oid graph_id,
                                 char *object_name,
                                 int object_id,
                                 bool id_field_exists)
@@ -222,7 +180,7 @@ int create_labels_from_csv_file(char *file_path,
     cr.header_row_length = 0;
     cr.curr_row_length = 0;
     cr.graph_name = graph_name;
-    cr.graph_oid = graph_oid;
+    cr.graph_id = graph_id;
     cr.object_name = object_name;
     cr.object_id = object_id;
     cr.id_field_exists = id_field_exists;
