@@ -19,34 +19,21 @@
 
 #include "postgres.h"
 
-#include "access/heapam.h"
-#include "access/xact.h"
-#include "parser/parse_node.h"
-#include "storage/lockdefs.h"
-#include "tcop/dest.h"
-#include "utils/builtins.h"
-#include "utils/lsyscache.h"
-#include "utils/rel.h"
-
-#include "catalog/ag_graph.h"
-#include "catalog/ag_label.h"
-#include "utils/agtype.h"
-#include "utils/graphid.h"
-
-#include "utils/load/ag_load_edges.h"
-#include "utils/load/ag_load_labels.h"
 #include "utils/load/age_load.h"
+#include "utils/load/ag_load_labels.h"
+#include "utils/load/ag_load_edges.h"
 
-agtype *create_empty_agtype(void)
+agtype* create_empty_agtype(void)
 {
     agtype* out;
     agtype_in_state result;
 
     memset(&result, 0, sizeof(agtype_in_state));
 
-    result.res = push_agtype_value(&result.parse_state, WAGT_BEGIN_OBJECT,
-                                   NULL);
-    result.res = push_agtype_value(&result.parse_state, WAGT_END_OBJECT, NULL);
+    result.res = push_agtype_value(&result.parse_state,
+                                   WAGT_BEGIN_OBJECT, NULL);
+    result.res = push_agtype_value(&result.parse_state,
+                                   WAGT_END_OBJECT, NULL);
 
     out = agtype_value_to_agtype(result.res);
     pfree_agtype_in_state(&result);
@@ -54,8 +41,8 @@ agtype *create_empty_agtype(void)
     return out;
 }
 
-agtype *create_agtype_from_list(char **header, char **fields, size_t fields_len,
-                                int64 vertex_id)
+agtype* create_agtype_from_list(char **header, char **fields,
+                                size_t fields_len, int64 vertex_id)
 {
     agtype* out;
     agtype_value* key_agtype;
@@ -65,8 +52,8 @@ agtype *create_agtype_from_list(char **header, char **fields, size_t fields_len,
 
     memset(&result, 0, sizeof(agtype_in_state));
 
-    result.res = push_agtype_value(&result.parse_state, WAGT_BEGIN_OBJECT,
-                                   NULL);
+    result.res = push_agtype_value(&result.parse_state,
+                                   WAGT_BEGIN_OBJECT, NULL);
 
     key_agtype = string_to_agtype_value("__id__");
     result.res = push_agtype_value(&result.parse_state,
@@ -119,13 +106,12 @@ agtype* create_agtype_from_list_i(char **header, char **fields,
     {
         return create_empty_agtype();
     }
-
     memset(&result, 0, sizeof(agtype_in_state));
 
-    result.res = push_agtype_value(&result.parse_state, WAGT_BEGIN_OBJECT,
-                                   NULL);
+    result.res = push_agtype_value(&result.parse_state,
+                                   WAGT_BEGIN_OBJECT, NULL);
 
-    for (i = start_index; i < fields_len; i++)
+    for (i = start_index; i<fields_len; i++)
     {
         key_agtype = string_to_agtype_value(header[i]);
         result.res = push_agtype_value(&result.parse_state,
@@ -149,9 +135,9 @@ agtype* create_agtype_from_list_i(char **header, char **fields,
     return out;
 }
 
-void insert_edge_simple(Oid graph_oid, char *label_name, graphid edge_id,
+void insert_edge_simple(Oid graph_id, char* label_name, graphid edge_id,
                         graphid start_id, graphid end_id,
-                        agtype *edge_properties)
+                        agtype* edge_properties)
 {
 
     Datum values[6];
@@ -160,7 +146,7 @@ void insert_edge_simple(Oid graph_oid, char *label_name, graphid edge_id,
     HeapTuple tuple;
 
     // Check if label provided exists as vertex label, then throw error
-    if (get_label_kind(label_name, graph_oid) == LABEL_KIND_VERTEX)
+    if (get_label_kind(label_name, graph_id) == LABEL_KIND_VERTEX)
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("label %s already exists as vertex label", label_name)));
@@ -171,19 +157,21 @@ void insert_edge_simple(Oid graph_oid, char *label_name, graphid edge_id,
     values[2] = GRAPHID_GET_DATUM(end_id);
     values[3] = AGTYPE_P_GET_DATUM((edge_properties));
 
-    label_relation = table_open(get_label_relation(label_name, graph_oid),
-                                RowExclusiveLock);
+    label_relation = heap_open(get_label_relation(label_name,
+                                                  graph_id),
+                               RowExclusiveLock);
 
     tuple = heap_form_tuple(RelationGetDescr(label_relation),
                             values, nulls);
     heap_insert(label_relation, tuple,
                 GetCurrentCommandId(true), 0, NULL);
-    table_close(label_relation, RowExclusiveLock);
+    heap_close(label_relation, RowExclusiveLock);
     CommandCounterIncrement();
 }
 
-void insert_vertex_simple(Oid graph_oid, char *label_name, graphid vertex_id,
-                          agtype *vertex_properties)
+void insert_vertex_simple(Oid graph_id, char* label_name,
+                          graphid vertex_id,
+                          agtype* vertex_properties)
 {
 
     Datum values[2];
@@ -192,7 +180,7 @@ void insert_vertex_simple(Oid graph_oid, char *label_name, graphid vertex_id,
     HeapTuple tuple;
 
     // Check if label provided exists as edge label, then throw error
-    if (get_label_kind(label_name, graph_oid) == LABEL_KIND_EDGE)
+    if (get_label_kind(label_name, graph_id) == LABEL_KIND_EDGE)
     {
         ereport(ERROR, (errcode(ERRCODE_INVALID_PARAMETER_VALUE),
                         errmsg("label %s already exists as edge label", label_name)));
@@ -201,26 +189,29 @@ void insert_vertex_simple(Oid graph_oid, char *label_name, graphid vertex_id,
     values[0] = GRAPHID_GET_DATUM(vertex_id);
     values[1] = AGTYPE_P_GET_DATUM((vertex_properties));
 
-    label_relation = table_open(get_label_relation(label_name, graph_oid),
-                                RowExclusiveLock);
+    label_relation = heap_open(get_label_relation(label_name,
+                                                  graph_id),
+                               RowExclusiveLock);
     tuple = heap_form_tuple(RelationGetDescr(label_relation),
                             values, nulls);
     heap_insert(label_relation, tuple,
                 GetCurrentCommandId(true), 0, NULL);
-    table_close(label_relation, RowExclusiveLock);
+    heap_close(label_relation, RowExclusiveLock);
     CommandCounterIncrement();
 }
 
+
 PG_FUNCTION_INFO_V1(load_labels_from_file);
 Datum load_labels_from_file(PG_FUNCTION_ARGS)
 {
+
     Name graph_name;
     Name label_name;
     text* file_path;
     char* graph_name_str;
     char* label_name_str;
     char* file_path_str;
-    Oid graph_oid;
+    Oid graph_id;
     int32 label_id;
     bool id_field_exists;
 
@@ -252,11 +243,12 @@ Datum load_labels_from_file(PG_FUNCTION_ARGS)
     label_name_str = NameStr(*label_name);
     file_path_str = text_to_cstring(file_path);
 
-    graph_oid = get_graph_oid(graph_name_str);
-    label_id = get_label_id(label_name_str, graph_oid);
+    graph_id = get_graph_oid(graph_name_str);
+    label_id = get_label_id(label_name_str, graph_id);
 
-    create_labels_from_csv_file(file_path_str, graph_name_str, graph_oid,
-                                label_name_str, label_id, id_field_exists);
+    create_labels_from_csv_file(file_path_str, graph_name_str,
+                                graph_id, label_name_str,
+                                label_id, id_field_exists);
     PG_RETURN_VOID();
 
 }
@@ -271,7 +263,7 @@ Datum load_edges_from_file(PG_FUNCTION_ARGS)
     char* graph_name_str;
     char* label_name_str;
     char* file_path_str;
-    Oid graph_oid;
+    Oid graph_id;
     int32 label_id;
 
     if (PG_ARGISNULL(0))
@@ -300,11 +292,11 @@ Datum load_edges_from_file(PG_FUNCTION_ARGS)
     label_name_str = NameStr(*label_name);
     file_path_str = text_to_cstring(file_path);
 
-    graph_oid = get_graph_oid(graph_name_str);
-    label_id = get_label_id(label_name_str, graph_oid);
+    graph_id = get_graph_oid(graph_name_str);
+    label_id = get_label_id(label_name_str, graph_id);
 
-    create_edges_from_csv_file(file_path_str, graph_name_str, graph_oid,
-                               label_name_str, label_id);
+    create_edges_from_csv_file(file_path_str, graph_name_str,
+                               graph_id, label_name_str, label_id);
     PG_RETURN_VOID();
 
 }
